package blackbox

import (
	"fmt"
	"io"

	"bytes"
	"sync"
	"time"

	"zks.com/gotcp"
	utils "zks.com/socketutils"
	coldcloud "zks.com/zcoldcloud"
)

//var SockeData = struct {
//	sync.RWMutex
//	m map[string][]byte
//}{m: make(map[string][]byte)}

type SockeBody struct {
	Time int64
	Body []byte
}

var SockeData = struct {
	sync.RWMutex
	m map[string]SockeBody
}{m: make(map[string]SockeBody)}

// Packet
type BBoxPacketDelegate struct {
	pLen  uint32
	pType uint32
	pData []byte
}
type blackboxddata struct {
	Code      string
	Datapoint coldcloud.DatapointColdCloud
}

func (p *BBoxPacketDelegate) Serialize() []byte {
	return p.pData
}

func (p *BBoxPacketDelegate) GetLen() uint32 {
	return p.pLen
}

func (p *BBoxPacketDelegate) GetTypeInt() uint32 {
	return p.pType
}

func (p *BBoxPacketDelegate) GetTypeString() string {
	return ""
}

func (p *BBoxPacketDelegate) GetData() []byte {
	return p.pData
}

func NewPacket(pType uint32, pData []byte) *gotcp.Packet {
	packet := new(gotcp.Packet)
	packet.Delegate = &BBoxPacketDelegate{
		pLen:  uint32(len(pData)),
		pType: pType,
		pData: pData,
	}
	return packet
}

type BBoxProtocol struct {
}

//通用
func (this *BBoxProtocol) ReadPacket(r io.Reader, MaxPacketLength uint32) (*gotcp.Packet, error) {
	data := make([]byte, MaxPacketLength) // 设定缓存空间
	readLengh, err := r.Read(data)
	//utils.InformationalWithFormat("request: % X", data[:readLengh])
	if err != nil { //EOF, or worse
		fmt.Println(1, err)
		return nil, err
	}
	if readLengh == 0 { // 连接可能已被客户端关闭
		fmt.Println("数据长度读取为零")
		return nil, gotcp.ReadPacketError
		//return nil, err
	} else {
		//fmt.Println("next execuse NewPacket")
		return NewPacket(1, data[:readLengh]), nil
	}

}
func (this *BBoxProtocol) ReadPacket_OK(r io.Reader, MaxPacketLength uint32) (*gotcp.Packet, error) {
	var fullBuf *bytes.Buffer = bytes.NewBuffer([]byte{}) //缓存区
	for {
		data := make([]byte, MaxPacketLength) // 设定缓存空间
		readLengh, err := r.Read(data)
		if err != nil { //EOF, or worse
			return nil, err
		}
		if readLengh == 0 { // 连接可能已被客户端关闭
			return nil, gotcp.ReadPacketError
		} else {
			//记录日志
			utils.InformationalWithFormat("% X", data[:readLengh])
			if (data[0] == 0x7E && data[readLengh-1] == 0x7E) || (data[0] == 0x8E && data[readLengh-1] == 0x8E) {
				if len(fullBuf.Bytes()) > 0 {
					fmt.Println("错误日志>黏包", fmt.Sprintf("% X", fullBuf.Bytes()))
				}
				return NewPacket(1, data[:readLengh]), nil
			} else {
				fullBuf.Write(data[:readLengh]) //写入数据到缓冲区
				if (fullBuf.Bytes()[0] == 0x7E && fullBuf.Bytes()[len(fullBuf.Bytes())-1] == 0x7E) || (fullBuf.Bytes()[0] == 0x8E && fullBuf.Bytes()[len(fullBuf.Bytes())-1] == 0x8E) {
					return NewPacket(1, fullBuf.Bytes()), nil
				} else if fullBuf.Bytes()[0] == 0x7E {
					continue
				}
			}
		}
	}
}

type BBoxConnDelegate struct {
	connectCount int
	closeCount   int
	messageCount int
}

func (this *BBoxConnDelegate) OnConnect(c *gotcp.Conn) bool {

	this.connectCount++
	Conns.Lock()
	fmt.Println("连接数：", len(Conns.m))
	Conns.Unlock()
	c.PutExtraData(this.connectCount)

	return true
}

// 正式使用
func (this *BBoxConnDelegate) OnMessage_OK(c *gotcp.Conn, p *gotcp.Packet) bool {
	fmt.Println("连接数：", c.GetRawConn().RemoteAddr())
	bs := p.GetData()
	utils.InformationalWithFormat("OnMessage:% X", bs)
	if len(bs) >= 2 {
		if bs[0] == 0x8E {
			go DoWork(c, bs)
		} else {
			data := []byte{}
			for i := 0; i < len(bs); i++ {
				data = append(data, bs[i])
				if i != 0 && bs[i] == 0x7E && len(data) > 2 {
					go DoWork(c, data)
					data = []byte{}
				}
			}
		}
	}
	return true
}

//记录socket通道存断开数据
func (this *BBoxConnDelegate) OnMessage(c *gotcp.Conn, p *gotcp.Packet) bool {
	bs := p.GetData()
	utils.InformationalWithFormat("OnMessage:% X", bs)
	if len(bs) >= 2 {
		data := []byte{}
		//正常数据有7E开头结束
		if bs[0] == 0x7E && bs[len(bs)-1] == 0x7E {
			SockeData.Lock()
			if len(SockeData.m[utils.ToStr(c.GetRawConn().RemoteAddr())].Body) > 0 { //map中有待发数据
				oldbs := SockeData.m[utils.ToStr(c.GetRawConn().RemoteAddr())].Body
				var newbs []byte
				rightdata := false
				for i := 0; i < len(oldbs); i++ {
					if len(newbs) == 0 && oldbs[i] == 0x7E && len(oldbs)-1 > i && oldbs[i+1] != 0x7E {
						rightdata = true
					}
					if rightdata == true {
						newbs = append(newbs, oldbs[i])
					}
					if len(newbs) > 5 && oldbs[i] == 0x7E { //获取黏包中有效数据
						rightdata = false
						go DoWork(c, newbs)
						newbs = []byte{}
					}
				}
				fmt.Println(c.GetRawConn().RemoteAddr(), "错误日志>黏包", fmt.Sprintf("% X", SockeData.m[utils.ToStr(c.GetRawConn().RemoteAddr())].Body))
				Writedata_v2(fmt.Sprintf("% X", SockeData.m[utils.ToStr(c.GetRawConn().RemoteAddr())].Body))
			}
			delete(SockeData.m, utils.ToStr(c.GetRawConn().RemoteAddr()))
			SockeData.Unlock()
			for i := 0; i < len(bs); i++ {
				data = append(data, bs[i])
				if i != 0 && bs[i] == 0x7E && len(data) > 2 {
					go DoWork(c, data)
					data = []byte{}
				}
			}
		} else if bs[0] == 0x8E && bs[len(bs)-1] == 0x8E { //平台下发命令
			go DoWork(c, bs)
		} else { //异常数据
			SockeData.Lock()
			bodylen := len(SockeData.m[utils.ToStr(c.GetRawConn().RemoteAddr())].Body)
			if bodylen > 0 && SockeData.m[utils.ToStr(c.GetRawConn().RemoteAddr())].Body[0] == 0x7E && bs[len(bs)-1] == 0x7E { //断包处理，map中有7E开头，新来数据有7E结尾
				okbody := append(SockeData.m[utils.ToStr(c.GetRawConn().RemoteAddr())].Body, bs...)
				delete(SockeData.m, utils.ToStr(c.GetRawConn().RemoteAddr()))
				SockeData.Unlock()
				for i := 0; i < len(okbody); i++ {
					data = append(data, okbody[i])
					if i != 0 && okbody[i] == 0x7E && len(data) > 2 {
						go DoWork(c, data)
						data = []byte{}
					}
				}
			} else if bodylen == 0 && bs[0] != 0x7E { //map中没有信息，新来数据没有7E开头（处理脏数据）
				var newbs []byte
				rightdata := false
				for i := 0; i < len(bs); i++ {
					if len(newbs) == 0 && bs[i] == 0x7E && len(bs)-1 > i && bs[i+1] != 0x7E {
						rightdata = true
					}
					if rightdata == true {
						newbs = append(newbs, bs[i])
					}

					if len(newbs) > 5 && bs[i] == 0x7E { //捕获带有脏数据有7E开头结尾的数据
						rightdata = false
						go DoWork(c, newbs)
						newbs = []byte{}
					}
				}
				//处理带有脏数据的有7E开头没有7E结尾的数据，去掉开头脏数据保留有效数据存入map
				if len(newbs) > 5 {
					SockeData.m[utils.ToStr(c.GetRawConn().RemoteAddr())] = SockeBody{Time: time.Now().Unix(), Body: newbs}
					fmt.Println(c.GetRawConn().RemoteAddr(), "错误日志>无7E开头(黏指令)", fmt.Sprintf("% X", bs))
				} else {
					fmt.Println(c.GetRawConn().RemoteAddr(), "错误日志>无7E开头", fmt.Sprintf("% X", bs))
				}
				SockeData.Unlock()
				Writedata_v2(fmt.Sprintf("% X", bs))
			} else { //断包处理后半段数据，（map中有信息，新来数据没有7E开头）？
				SockeData.m[utils.ToStr(c.GetRawConn().RemoteAddr())] = SockeBody{Time: time.Now().Unix(), Body: append(SockeData.m[utils.ToStr(c.GetRawConn().RemoteAddr())].Body, bs...)}
				SockeData.Unlock()
			}
		}
	}
	return true
}
func (this *BBoxConnDelegate) OnMessage1(c *gotcp.Conn, p *gotcp.Packet) bool {
	fmt.Println("连接数：", c.GetRawConn().RemoteAddr())
	bs := p.GetData()
	utils.InformationalWithFormat("OnMessage:% X", bs)
	if len(bs) >= 2 {
		if bs[0] == 0x8E {
			go DoWork(c, bs)
		} else {
			data := []byte{}
			for i := 0; i < len(bs); i++ {
				data = append(data, bs[i])
				if i != 0 && bs[i] == 0x7E && len(data) > 2 {
					go DoWork(c, data)
					data = []byte{}
				}
			}
		}
	}
	return true
}
func (this *BBoxConnDelegate) OnClose(c *gotcp.Conn) {
	defer func() {
		if r := recover(); r != nil {
			fmt.Println("OnClose Recovered in f", r)
		}
	}()
	this.closeCount++
	Conns.Lock()
	for k, v := range Conns.m {
		if v == c {
			fmt.Println(k + "已断开")
			delete(Conns.m, k)
			fmt.Println("剩余连接数：", len(Conns.m))
		}
	}
	Conns.Unlock()
	SockeData.Lock()
	if len(SockeData.m[utils.ToStr(c.GetRawConn().RemoteAddr())].Body) > 0 {
		oldbs := SockeData.m[utils.ToStr(c.GetRawConn().RemoteAddr())].Body
		var newbs []byte
		rightdata := false
		for i := 0; i < len(oldbs); i++ {
			if len(newbs) == 0 && oldbs[i] == 0x7E && len(oldbs)-1 > i && oldbs[i+1] != 0x7E {
				rightdata = true
			}
			if rightdata == true {
				newbs = append(newbs, oldbs[i])
			}
			if len(newbs) > 5 && oldbs[i] == 0x7E {
				rightdata = false
				go DoWork(c, newbs)
				newbs = []byte{}
			}
		}
		fmt.Println("错误日志>连接断开数据包丢失", fmt.Sprintf("% X", SockeData.m[utils.ToStr(c.GetRawConn().RemoteAddr())].Body))
		Writedata_v2(fmt.Sprintf("% X", SockeData.m[utils.ToStr(c.GetRawConn().RemoteAddr())].Body))
	}
	delete(SockeData.m, utils.ToStr(c.GetRawConn().RemoteAddr()))
	fmt.Println("统计黏包socket", len(SockeData.m))
	SockeData.Unlock()
	fmt.Printf("OnClose[%s][***%v***]\n")
}

func (this *BBoxConnDelegate) OnIOError(c *gotcp.Conn, err error) {
	fmt.Printf("OnIOError[%s][***%v***]:[%v]\n", c.GetRawConn().RemoteAddr(), c.GetExtraData().(int), err)
}
