/*更新记录
20-5-7-23 zh 增加深低温校准（温度在-195到-190之间时 将温度改为 196）
20-5-9-8 zh 增加深低温校准（温度小于-197时 将温度改为 196 小数点后为随机数）
20-5-10-13 zh 修改校准规则，存储前一条数据到缓存boltdb中
*/

package blackbox

import (
	"fmt"
	"strconv"
	"sync"
	"time"

	"encoding/json"

	"zks.com/gotcp"
	zksutils "zks.com/zUtils"
	coldcloud "zks.com/zcoldcloud"
)

var format string = "2006-1-2 15:4:5"
var Conns = struct {
	sync.RWMutex
	m map[string]*gotcp.Conn
}{m: make(map[string]*gotcp.Conn)}
var Conns2004 = struct {
	sync.RWMutex
	m map[string]Status2004
}{m: make(map[string]Status2004)}

type Status2004 struct {
	Time  int64
	Count int64
}
type Status struct {
	Status  int    `json:"status"`
	Id      string `json:"id"`
	Message string `json:"message"`
}

func DoWork(c *gotcp.Conn, command []byte) error {
	fmt.Println("DoWork", time.Now(), fmt.Sprintf("%X", command))
	defer func() {
		if r := recover(); r != nil {
			fmt.Println("Recovered in f", r)
		}
	}()
	if 0x7E != command[0] || 0x7E != command[len(command)-1] {
		if 0x8E == command[0] && 0x8E == command[len(command)-1] {
			go ControlDevice(c, command)
		}
	} else {
		//创建新的byte数组
		newcom := make([]byte, 0)
		//循环便利原数组,去掉首尾的0x7E,把数据中的0x7D 0x01转义为0x7D,0x7D 0x02转义为0x7E
		for i := 1; i < len(command)-1; i++ {

			if command[i] == 0x7D && command[i+1] == 0x01 {
				newcom = append(newcom, 0x7D)
				i++
			} else if command[i] == 0x7D && command[i+1] == 0x02 {
				newcom = append(newcom, 0x7E)
				i++
			} else {
				newcom = append(newcom, command[i])
			}
		}
		if len(newcom) >= 13 {
			bytejh := yihuo(newcom[0 : len(newcom)-1])
			//fmt.Println("校验码： ", bytejh)
			//fmt.Println("原校验码： ", newcom[len(newcom)-1])
			//消息ID
			msgid := fmt.Sprintf("%X", newcom[0:2])
			//fmt.Println("消息ID ", msgid)
			//消息体属性
			msgattr := newcom[2:4]
			str := ByteToBinaryString(msgattr[0])
			str1 := ByteToBinaryString(msgattr[1])
			//消息体长度
			msglength := btox(Substr(str+str1, len(str+str1)-11, 11))
			//fmt.Println("标记消息体长度 ", msglength)
			//实际消息体长度
			msgsjcd := len(newcom[12:]) - 1
			msglengthint, _ := strconv.Atoi(msglength)
			//fmt.Println("实际消息体长度 ", msgsjcd)
			//网关ID
			msggatewayid := fmt.Sprintf("%X", newcom[4:8])
			fmt.Println("网关ID ", msggatewayid)
			coldcloud.UpdateBoxIpAndPort(msggatewayid, coldcloud.IpAndPortStruct{Config.Service.Ipandport, time.Now()})
			//UUID
			//msguuid := fmt.Sprintf("%X", newcom[8:10])
			//fmt.Println("msguuid  ", msguuid) //msguuid
			//消息流水号
			msgliusidx := fmt.Sprintf("%X", newcom[10:12])
			msgliusid, _ := strconv.ParseInt(msgliusidx, 16, 64)
			//fmt.Println("消息流水号 ", msgliusid)
			//消息包总数
			//var msgtotal int64
			//fmt.Println(msgtotal) //msgtotal
			//消息包序号
			//var msgnum int64
			//fmt.Println(msgnum) //msgnum
			if bytejh == newcom[len(newcom)-1] && msglengthint == msgsjcd {
				//A029下发报警给报警器
				if msgid != "A029" {
					Conns.Lock()
					Conns.m[msggatewayid] = c
					Conns.Unlock()
				}
				switch msgid {
				case "2004", "2024", "2025", "2010", "2011":
					var datacom []byte
					datacom = newcom[12 : 12+msgsjcd]
					fmt.Println(fmt.Sprintf("% X", datacom))
					GetChannelValue(c, datacom, command, msgid, msggatewayid, msgliusid)
				case "A029":
					Conns.Lock()
					if v, ok := Conns.m[msggatewayid]; ok {
						fmt.Println("A029发送", fmt.Sprintf("%X", command))
						go v.WritePacket(NewPacket(3, command))
					}
					Conns.Unlock()
				}
				DParamSet(c, msggatewayid)
				ShutdownOrBoot(c, msggatewayid)
			}
		}
		Writedata_v2(fmt.Sprintf("% X", command))
	}
	return nil
}

//设置参数
func DParamSet(c *gotcp.Conn, msggatewayid string) {
	paramset, _ := coldcloud.GetReverseControlDParam("CTL" + msggatewayid)
	if paramset.Id != 0 && len(msggatewayid) >= 8 {
		head := []byte{0xA0, 0x02, 0x00}
		sendms := []byte{}
		for _, v := range paramset.SetValue {
			if time.Now().Unix()-v.CreateOn.Unix() > (60 * 30) {
				sendms = []byte{}
				break
			}
			switch v.Dtype {
			case 1:
				zksint, _ := zksutils.StrTo(v.Value).Int()
				sendms = append(sendms, byte(v.Dtype))
				sendms = append(sendms, Valuebyte4(zksint)...)
			case 2:
				zksint, _ := zksutils.StrTo(v.Value).Int()
				sendms = append(sendms, byte(v.Dtype))
				fmt.Println(fmt.Sprintf("% X", sendms))
				sendms = append(sendms, Valuebyte4(zksint)...)
			case 3:
				zksfloat, _ := zksutils.StrTo(v.Value).Float32()
				zksint := int(zksfloat * 100)
				sendms = append(sendms, byte(v.Dtype))
				sendms = append(sendms, byte(v.Port))
				if zksint >= 0 {
					sendms = append(sendms, Valuebyte2(zksint)...)
				} else {
					sendms = append(sendms, Bumatobyte2(zksint)...)
				}
			case 4:
				zksfloat, _ := zksutils.StrTo(v.Value).Float32()
				zksint := int(zksfloat * 100)
				sendms = append(sendms, byte(v.Dtype))
				sendms = append(sendms, byte(v.Port))
				if zksint >= 0 {
					sendms = append(sendms, Valuebyte2(zksint)...)
				} else {
					sendms = append(sendms, Bumatobyte2(zksint)...)
				}
			case 11:
				name := []byte(CodertoGBK(zksutils.Substr(v.Value, 0, 10)))
				sendms = append(sendms, byte(v.Dtype))
				sendms = append(sendms, byte(len(name)))
				sendms = append(sendms, name...)
			case 27:
				zksfloat, _ := zksutils.StrTo(v.Value).Float32()
				zksint := int(zksfloat * 100)
				sendms = append(sendms, byte(v.Dtype))
				sendms = append(sendms, byte(v.Port))
				if zksint >= 0 {
					sendms = append(sendms, Valuebyte2(zksint)...)
				} else {
					sendms = append(sendms, Bumatobyte2(zksint)...)
				}
			case 28: //湿度校正
				zksfloat, _ := zksutils.StrTo(v.Value).Float32()
				zksint := int(zksfloat * 100)
				sendms = append(sendms, byte(v.Dtype))
				sendms = append(sendms, byte(v.Port))
				if zksint >= 0 {
					sendms = append(sendms, Valuebyte2(zksint)...)
				} else {
					sendms = append(sendms, Bumatobyte2(zksint)...)
				}
			}
		}
		if len(sendms) == 0 {
			//Apipost(Config.Send.Waterdrop.Service+"/dparamset/setv2?mainserial="+msggatewayid+"&num="+zksutils.ToStr(paramset.ControlNum)+"&state=2", "POST", nil)
			var dparamset coldcloud.DParamSet
			coldcloud.UpdateCTLDParamSet("CTL"+msggatewayid, dparamset)
		} else {
			mac1, _ := strconv.ParseInt(msggatewayid[0:2], 16, 64)
			mac2, _ := strconv.ParseInt(msggatewayid[2:4], 16, 64)
			mac3, _ := strconv.ParseInt(msggatewayid[4:6], 16, 64)
			mac4, _ := strconv.ParseInt(msggatewayid[6:8], 16, 64)
			//		ctlnum, _ := coldcloud.GetReverseControlCtlSNum("CTL" + msggatewayid)
			//		numint, _ := zksutils.StrTo(ctlnum).Int()
			head = append(head, byte(len(sendms)), byte(mac1), byte(mac2), byte(mac3), byte(mac4), 0x00, 0x00)
			head = append(head, Valuebyte2(paramset.ControlNum)...)
			sendms = append(head, sendms...)
			sendms = append(sendms, yihuo(sendms))
			send := []byte{0x7E}
			for _, v := range sendms {
				if v == 0x7E {
					send = append(send, 0x7D, 0x02)
				} else if v == 0x7D {
					send = append(send, 0x7D, 0x01)
				} else {
					send = append(send, v)
				}
			}
			send = append(send, 0x7E)
			fmt.Println("反向控制命令", fmt.Sprintf("% X", send))
			c.WritePacket(NewPacket(3, send))
		}
	}
	//设置飞行模式
	lightset, _ := coldcloud.GetReverseControlSetLight("CTL" + msggatewayid)
	if lightset.SetType == 1 && len(msggatewayid) >= 8 && lightset.RequestState == 0 {
		if time.Now().Unix() > (lightset.OccurTime.Unix() - 120) {
			var status Status
			if time.Now().Unix() < lightset.CloseTime.Unix() || lightset.RemainMin == 0 {
				head := []byte{0xA0, 0x0B, 0x00}
				sendms := []byte{}
				mac1, _ := strconv.ParseInt(msggatewayid[0:2], 16, 64)
				mac2, _ := strconv.ParseInt(msggatewayid[2:4], 16, 64)
				mac3, _ := strconv.ParseInt(msggatewayid[4:6], 16, 64)
				mac4, _ := strconv.ParseInt(msggatewayid[6:8], 16, 64)
				head = append(head, 0x02, byte(mac1), byte(mac2), byte(mac3), byte(mac4), 0x00, 0x00)
				head = append(head, Valuebyte2(lightset.ControlNumber)...)
				sendms = append(sendms, Valuebyte2(lightset.RemainMin)...)
				sendms = append(head, sendms...)
				sendms = append(sendms, yihuo(sendms))
				send := []byte{0x7E}
				for _, v := range sendms {
					if v == 0x7E {
						send = append(send, 0x7D, 0x02)
					} else if v == 0x7D {
						send = append(send, 0x7D, 0x01)
					} else {
						send = append(send, v)
					}
				}
				send = append(send, 0x7E)
				fmt.Println("设置飞行模式", fmt.Sprintf("% X", send))
				c.WritePacket(NewPacket(3, send))
				json.Unmarshal(Apipost(Config.Send.Waterdrop.Service+"/devicestate/state?mainserial="+msggatewayid+"&controlnumber="+zksutils.ToStr(lightset.ControlNumber)+"&settype=1&requeststate=1&requesttime="+time.Now().Format("2006-01-02 15:04:05"), "PUT", nil), &status)
				if status.Status == 1 {
					lightset.RequestState = 1
					lightset.OccurTime = time.Now()
					coldcloud.UpdateCtlSetLight("CTL"+msggatewayid, lightset)
				}
			} else {
				json.Unmarshal(Apipost(Config.Send.Waterdrop.Service+"/devicestate/state?mainserial="+msggatewayid+"&controlnumber="+zksutils.ToStr(lightset.ControlNumber)+"&settype=1&requeststate=4&requesttime="+time.Now().Format("2006-01-02 15:04:05"), "PUT", nil), &status)
				if status.Status == 1 {
					lightset.RequestState = 4
					lightset.OccurTime = time.Now()
					coldcloud.UpdateCtlSetLight("CTL"+msggatewayid, lightset)
				}
			}

		}
	}
	//远程升级
	dupgrade, err := coldcloud.GetReverseControlDUpgrade("CTL" + msggatewayid)
	if err == nil && len(dupgrade.Data) > 0 {
		if time.Now().Unix()-dupgrade.Time < 1800 {
			mac1, _ := strconv.ParseInt(msggatewayid[0:2], 16, 64)
			mac2, _ := strconv.ParseInt(msggatewayid[2:4], 16, 64)
			mac3, _ := strconv.ParseInt(msggatewayid[4:6], 16, 64)
			mac4, _ := strconv.ParseInt(msggatewayid[6:8], 16, 64)
			sendms := []byte{0xA0, 0x0A, 0x00, 0x00, byte(mac1), byte(mac2), byte(mac3), byte(mac4), 0x00, 0x00, 0x00, 0x00}
			sendms = append(sendms, yihuo(sendms))
			send := []byte{0x7E}
			for _, v := range sendms {
				if v == 0x7E {
					send = append(send, 0x7D, 0x02)
				} else if v == 0x7D {
					send = append(send, 0x7D, 0x01)
				} else {
					send = append(send, v)
				}
			}
			send = append(send, 0x7E)
			err := c.WritePacket(NewPacket(3, send))
			fmt.Println(err, "缓存远程升级", fmt.Sprintf("% X", send))
		}
	}
}

//平台发送多条同一推送报警信息(暂时不用)
func ServiceToTriggerMsg(c *gotcp.Conn, msggatewayid string) {
	cachevalue, err := coldcloud.GetReverseControlTriggerMac("CTL" + msggatewayid)
	//cachevalue, err := coldcloud.GetTriggerMac("TMac" + msggatewayid)
	if err == nil && len(cachevalue.TriggerMac) > 0 {
		fmt.Println("平台发送报警信息")
		var data []byte
		for _, v := range cachevalue.TriggerMac {
			mac1, _ := strconv.ParseInt(v.MainMac[0:2], 16, 64)
			mac2, _ := strconv.ParseInt(v.MainMac[2:4], 16, 64)
			mac3, _ := strconv.ParseInt(v.MainMac[4:6], 16, 64)
			mac4, _ := strconv.ParseInt(v.MainMac[6:8], 16, 64)
			port, _ := strconv.ParseUint(v.Port, 16, 64)
			lastdata, err1 := coldcloud.GetTriggerHistoryLast("trigger_c" + v.Mac)
			macdata := []byte{}
			if err1 == nil {
				for _, v1 := range lastdata {
					if v1.IsSend == 2 && v1.ActionType == "alertor" && v1.SendTo == msggatewayid {
						if v1.EventFiled == "temperature" {
							macdata = []byte{byte(mac1), byte(mac2), byte(mac3), byte(mac4), byte(port), byte(1)}
							macdata = append(macdata, Bumabyte4(int64(v1.Value*100))...)
						} else if v1.EventFiled == "humidity" {
							macdata = []byte{byte(mac1), byte(mac2), byte(mac3), byte(mac4), byte(port), byte(2)}
							macdata = append(macdata, Bumabyte4(int64(v1.Value*100))...)
						}
					}
				}
			}
			if len(macdata) > 0 {
				data = append(data, macdata...)
			}
		}
		var sendms []byte
		if len(data) > 0 && len(data) < 100 {
			mac1, _ := strconv.ParseInt(msggatewayid[0:2], 16, 64)
			mac2, _ := strconv.ParseInt(msggatewayid[2:4], 16, 64)
			mac3, _ := strconv.ParseInt(msggatewayid[4:6], 16, 64)
			mac4, _ := strconv.ParseInt(msggatewayid[6:8], 16, 64)
			ctlnum, _ := coldcloud.GetReverseControlCtlSNum("CTL" + msggatewayid)
			numint, _ := zksutils.StrTo(ctlnum).Int()
			sendms = []byte{0xA0, 0x03, 0x00, byte(len(data)), byte(mac1), byte(mac2), byte(mac3), byte(mac4), 0x00, 0x00}
			sendms = append(sendms, Valuebyte2(numint)...)
			sendms = append(sendms, data...)
			sendms = append(sendms, yihuo(sendms))
			send := []byte{0x7E}
			for _, v := range sendms {
				if v == 0x7E {
					send = append(send, 0x7D, 0x02)
				} else if v == 0x7D {
					send = append(send, 0x7D, 0x01)
				} else {
					send = append(send, v)
				}
			}
			send = append(send, 0x7E)
			fmt.Println("平台发送报警信息", fmt.Sprintf("% X", send))
			c.WritePacket(NewPacket(3, send))
		}
	}
}

//平台推送单条报警信息
func ServiceToTriggerSingleMsg(c *gotcp.Conn, msggatewayid string) {
	cachevalue, err := coldcloud.GetReverseControlTriggerMac("CTL" + msggatewayid)
	if err == nil && len(cachevalue.TriggerMac) > 0 {
		fmt.Println("平台发送报警信息", len(cachevalue.TriggerMac))
		for _, v := range cachevalue.TriggerMac {
			if len(v.MainMac) >= 8 && len(msggatewayid) >= 8 {
				mac1, _ := strconv.ParseInt(v.MainMac[0:2], 16, 64)
				mac2, _ := strconv.ParseInt(v.MainMac[2:4], 16, 64)
				mac3, _ := strconv.ParseInt(v.MainMac[4:6], 16, 64)
				mac4, _ := strconv.ParseInt(v.MainMac[6:8], 16, 64)
				port, _ := strconv.ParseUint(v.Port, 16, 64)
				lastdata, err1 := coldcloud.GetTriggerHistoryLast("trigger_c" + v.Mac)
				if err1 == nil {
					for _, v1 := range lastdata {
						if v1.IsSend == 2 && v1.ActionType == "alertor" && v1.SendTo == msggatewayid {
							macdata := []byte{}
							if v1.EventFiled == "temperature" {
								macdata = []byte{byte(mac1), byte(mac2), byte(mac3), byte(mac4), byte(port), byte(1)}
								macdata = append(macdata, Bumabyte4(int64(v1.Value*100))...)
								t := v1.AlarmOn.Format("20060102150405")
								uyear, _ := strconv.ParseUint(t[2:4], 16, 0)
								umonth, _ := strconv.ParseUint(t[4:6], 16, 0)
								uday, _ := strconv.ParseUint(t[6:8], 16, 0)
								uhour, _ := strconv.ParseUint(t[8:10], 16, 0)
								uminute, _ := strconv.ParseUint(t[10:12], 16, 0)
								usecond, _ := strconv.ParseUint(t[12:], 16, 0)
								macdata = append(macdata, byte(uyear), byte(umonth), byte(uday), byte(uhour), byte(uminute), byte(usecond))
								var sendms []byte
								mac1, _ := strconv.ParseInt(msggatewayid[0:2], 16, 64)
								mac2, _ := strconv.ParseInt(msggatewayid[2:4], 16, 64)
								mac3, _ := strconv.ParseInt(msggatewayid[4:6], 16, 64)
								mac4, _ := strconv.ParseInt(msggatewayid[6:8], 16, 64)
								sendms = []byte{0xA0, 0x03, 0x00, byte(len(macdata)), byte(mac1), byte(mac2), byte(mac3), byte(mac4), 0x00, 0x00}
								sendms = append(sendms, Valuebyte2(v1.ControlNum)...)
								sendms = append(sendms, macdata...)
								sendms = append(sendms, yihuo(sendms))
								send := []byte{0x7E}
								for _, v := range sendms {
									if v == 0x7E {
										send = append(send, 0x7D, 0x02)
									} else if v == 0x7D {
										send = append(send, 0x7D, 0x01)
									} else {
										send = append(send, v)
									}
								}
								send = append(send, 0x7E)
								fmt.Println(time.Now(), "流水号", v1.ControlNum, "平台发送报警信息温度 ", fmt.Sprintf("% X", send))
								c.WritePacket(NewPacket(3, send))
							} else if v1.EventFiled == "humidity" {
								macdata = []byte{byte(mac1), byte(mac2), byte(mac3), byte(mac4), byte(port), byte(2)}
								macdata = append(macdata, Bumabyte4(int64(v1.Value*100))...)
								t := v1.AlarmOn.Format("20060102150405")
								uyear, _ := strconv.ParseUint(t[2:4], 16, 0)
								umonth, _ := strconv.ParseUint(t[4:6], 16, 0)
								uday, _ := strconv.ParseUint(t[6:8], 16, 0)
								uhour, _ := strconv.ParseUint(t[8:10], 16, 0)
								uminute, _ := strconv.ParseUint(t[10:12], 16, 0)
								usecond, _ := strconv.ParseUint(t[12:], 16, 0)
								macdata = append(macdata, byte(uyear), byte(umonth), byte(uday), byte(uhour), byte(uminute), byte(usecond))
								var sendms []byte
								mac1, _ := strconv.ParseInt(msggatewayid[0:2], 16, 64)
								mac2, _ := strconv.ParseInt(msggatewayid[2:4], 16, 64)
								mac3, _ := strconv.ParseInt(msggatewayid[4:6], 16, 64)
								mac4, _ := strconv.ParseInt(msggatewayid[6:8], 16, 64)
								ctlnum, _ := coldcloud.GetReverseControlCtlSNum("CTL" + msggatewayid)
								numint, _ := zksutils.StrTo(ctlnum).Int()
								sendms = []byte{0xA0, 0x03, 0x00, byte(len(macdata)), byte(mac1), byte(mac2), byte(mac3), byte(mac4), 0x00, 0x00}
								sendms = append(sendms, Valuebyte2(numint)...)
								sendms = append(sendms, macdata...)
								sendms = append(sendms, yihuo(sendms))
								send := []byte{0x7E}
								for _, v := range sendms {
									if v == 0x7E {
										send = append(send, 0x7D, 0x02)
									} else if v == 0x7D {
										send = append(send, 0x7D, 0x01)
									} else {
										send = append(send, v)
									}
								}
								send = append(send, 0x7E)
								fmt.Println(time.Now(), "流水号", "平台发送报警信息湿度", fmt.Sprintf("% X", send))
								c.WritePacket(NewPacket(3, send))

							}
						}
					}
				}
			}

		}
	}
}

//平台远程控制设备（关机、重启）
func ShutdownOrBoot(c *gotcp.Conn, msggatewayid string) {
	//设置关机、重启
	shutdown, _ := coldcloud.GetReverseControlShutDown("CTL" + msggatewayid)
	if len(msggatewayid) >= 8 && len(shutdown.Serial) >= 8 && shutdown.ControlType != 0 {
		if time.Now().Unix() > (shutdown.CreatTime.Unix() - 120) {
			head := []byte{0xA0, 0x05, 0x00}
			sendms := []byte{}
			mac1, _ := strconv.ParseInt(msggatewayid[0:2], 16, 64)
			mac2, _ := strconv.ParseInt(msggatewayid[2:4], 16, 64)
			mac3, _ := strconv.ParseInt(msggatewayid[4:6], 16, 64)
			mac4, _ := strconv.ParseInt(msggatewayid[6:8], 16, 64)
			head = append(head, 0x05, byte(mac1), byte(mac2), byte(mac3), byte(mac4), 0x00, 0x00)
			head = append(head, Valuebyte2(shutdown.ControlNumber)...)
			mac1, _ = strconv.ParseInt(shutdown.Serial[0:2], 16, 64)
			mac2, _ = strconv.ParseInt(shutdown.Serial[2:4], 16, 64)
			mac3, _ = strconv.ParseInt(shutdown.Serial[4:6], 16, 64)
			mac4, _ = strconv.ParseInt(shutdown.Serial[6:8], 16, 64)
			sendms = append(sendms, byte(mac1), byte(mac2), byte(mac3), byte(mac4))
			sendms = append(sendms, byte(shutdown.ControlType))
			sendms = append(head, sendms...)
			sendms = append(sendms, yihuo(sendms))
			send := []byte{0x7E}
			for _, v := range sendms {
				if v == 0x7E {
					send = append(send, 0x7D, 0x02)
				} else if v == 0x7D {
					send = append(send, 0x7D, 0x01)
				} else {
					send = append(send, v)
				}
			}
			send = append(send, 0x7E)
			fmt.Println("设置关机重启", fmt.Sprintf("% X", send))
			c.WritePacket(NewPacket(3, send))
			Apipost(Config.Send.Waterdrop.Service+"/devicestate/state?mainserial="+msggatewayid+"&controlnumber="+zksutils.ToStr(shutdown.ControlNumber)+"&requeststate=1&requesttime="+time.Now().Format("2006-01-02 15:04:05"), "PUT", nil)
		}
	}
}

//func ControlDevice(c *gotcp.Conn, command []byte) {
//	fmt.Println("实时发送", len(Conns.m))
//	Conns.Lock()
//	for k, v := range Conns.m {
//		if len(command) > 7 && k == fmt.Sprintf("%X", command[1:5]) {
//			switch fmt.Sprintf("%X", command[5:7]) {
//			case "A002":
//				fmt.Println("设置设备参数进入", k)
//				DParamSet(v, k)
//			case "A003":
//				fmt.Println("报警推送")
//				ServiceToTriggerSingleMsg(v, k)
//			case "A005":
//				fmt.Println("远程控制设备")
//			case "A00A":
//				mac1, _ := strconv.ParseInt(k[0:2], 16, 64)
//				mac2, _ := strconv.ParseInt(k[2:4], 16, 64)
//				mac3, _ := strconv.ParseInt(k[4:6], 16, 64)
//				mac4, _ := strconv.ParseInt(k[6:8], 16, 64)
//				//ctlnum, _ := coldcloud.GetReverseControlCtlSNum("CTL" + k)
//				//numint, _ := zksutils.StrTo(ctlnum).Int()
//				sendms := []byte{0xA0, 0x0A, 0x00, 0x00, byte(mac1), byte(mac2), byte(mac3), byte(mac4), 0x00, 0x00, 0x00, 0x00}
//				//sendms = append(sendms, Valuebyte2(numint)...)
//				sendms = append(sendms, yihuo(sendms))
//				send := []byte{0x7E}
//				for _, v := range sendms {
//					if v == 0x7E {
//						send = append(send, 0X7D, 0X02)
//					} else if v == 0x7D {
//						send = append(send, 0X7D, 0X01)
//					} else {
//						send = append(send, v)
//					}
//				}
//				send = append(send, 0x7E)
//				err := v.WritePacket(NewPacket(3, send))
//				fmt.Println(err, "远程升级", fmt.Sprintf("% X", send))
//			}
//		}
//	}
//	Conns.Unlock()
//}
func ControlDevice(c *gotcp.Conn, command []byte) {
	if len(command) > 7 {
		k := fmt.Sprintf("%X", command[1:5])
		Conns.Lock()
		if v, ok := Conns.m[k]; ok {
			Conns.Unlock()
			switch fmt.Sprintf("%X", command[5:7]) {
			case "A002":
				fmt.Println("设置设备参数进入", k)
				DParamSet(v, k)
			case "A003":
				fmt.Println("报警推送")
				ServiceToTriggerSingleMsg(v, k)
			case "A005":
				fmt.Println("远程控制设备")
			case "A00A":
				mac1, _ := strconv.ParseInt(k[0:2], 16, 64)
				mac2, _ := strconv.ParseInt(k[2:4], 16, 64)
				mac3, _ := strconv.ParseInt(k[4:6], 16, 64)
				mac4, _ := strconv.ParseInt(k[6:8], 16, 64)
				sendms := []byte{0xA0, 0x0A, 0x00, 0x00, byte(mac1), byte(mac2), byte(mac3), byte(mac4), 0x00, 0x00, 0x00, 0x00}
				sendms = append(sendms, yihuo(sendms))
				send := []byte{0x7E}
				for _, v := range sendms {
					if v == 0x7E {
						send = append(send, 0x7D, 0x02)
					} else if v == 0x7D {
						send = append(send, 0x7D, 0x01)
					} else {
						send = append(send, v)
					}
				}
				send = append(send, 0x7E)
				err := v.WritePacket(NewPacket(3, send))
				fmt.Println(err, "远程升级", fmt.Sprintf("% X", send))
			}
		} else {
			Conns.Unlock()
			fmt.Println("Key Not Found")
		}
	}
}
